@@ -23,20 +23,24 @@
 #include "mozVoikkoSpell.hxx"
 #include <nsComponentManagerUtils.h>
 #include <nsCOMPtr.h>
-#include <nsICategoryManager.h>
 #include <nsIConsoleService.h>
 #include <nsIFile.h>
 #include <nsILocalFile.h>
-#include <nsIXULRuntime.h>
 #include <nsServiceManagerUtils.h>
 #include <nsStringAPI.h>
+#include <nsISupports.h>
 #include <prerror.h>
 
-#if defined(__linux__)
+#ifndef SYSTEM_LIBVOIKKO
+#include "nsIMozvoikkoHelper.h"
+
+#define MOZVOIKKO_HELPER_CONTRACTID "@mozilla.org/mozvoikkohelper;1"
+#endif
+
+#if defined(__linux__) || defined(__GLIBC__)
 
 const char *preloadLibNames[] =
 {
-    "libmalaga.so.7",
     "libvoikko.so.1"
 };
 
@@ -46,21 +50,16 @@ const char * libvoikkoName = "libvoikko.so.1";
 
 const char *preloadLibNames[] =
 {
-    "libmalaga.7.0.0.dylib",
-    "libvoikko.1.7.0.dylib"
+    "libvoikko.1.dylib"
 };
 
-const char * libvoikkoName = "libvoikko.1.7.0.dylib";
+const char * libvoikkoName = "libvoikko.1.dylib";
 
 #elif defined(_WIN32)
 
 
 const char *preloadLibNames[] =
 {
-    "iconv.dll",
-    "intl.dll",
-    "libglib-2.0-0.dll",
-    "malaga.dll",
     "libvoikko-1.dll"
 };
 
@@ -127,6 +126,8 @@ nsCString prGetErrorText(void)
     return msg;
 }
 
+#ifndef SYSTEM_LIBVOIKKO
+
 nsresult checkLeafNameAndGetParent(nsIFile *file, nsIFile **parent,
     const nsACString &expectedLeafName)
 {
@@ -200,48 +201,41 @@ nsresult findFileInSubdirectory(nsIFile *dir, nsIFile **result,
     return rv;
 }
 
+/*
+ Doesn't return the mozvoikko library but a helper component's nsIFile
+ which is located at the same directory
+ */
 nsresult getMozVoikkoLibrary(nsIFile **file)
 {
     nsresult rv;
-    char *mozVoikkoLibPathC;
-  
+	nsCString path;
+	
     *file = nsnull;
-
-    nsCOMPtr<nsICategoryManager> catMgr =
-        do_GetService(NS_CATEGORYMANAGER_CONTRACTID);
-
-    if (!catMgr)
-    {
-        logMessage("Failed to get nsICategoryManager");
-        return NS_ERROR_FAILURE;
-    }
-
-    rv = catMgr->GetCategoryEntry("spell-check-engine", MOZ_VOIKKOSPELL_CONTRACTID,
-        &mozVoikkoLibPathC);
-    if (NS_FAILED(rv)) 
-    {
-        logMessage("Failed to get category entry for mozVoikko");
-        return rv;
-    }
-
-    nsCString path(mozVoikkoLibPathC);
-
-    nsCOMPtr<nsILocalFile> mozVoikkoLib;
-    mozVoikkoLib = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv);
-    if (!mozVoikkoLib)
-    {
-        logMessage("Failed to create instance of nsILoclFile");
-        return NS_ERROR_FAILURE;
-    }
-
-    rv = mozVoikkoLib->InitWithPath(NS_ConvertUTF8toUTF16(path));
-    if (NS_FAILED(rv))
-    {
-        logMessage("Failed to set path for nsILocalFile");
-        return rv;
-    }
-
-    rv = mozVoikkoLib->Clone(file);
+	
+	nsCOMPtr<nsIMozvoikkoHelper> mozvoikkoHelper = do_GetService(MOZVOIKKO_HELPER_CONTRACTID);
+	if (!mozvoikkoHelper)
+	{
+		logMessage("Failed to create instance of %s. (rv = 0x%08x)\n", MOZVOIKKO_HELPER_CONTRACTID);
+		return NS_ERROR_UNEXPECTED;
+	}
+	
+	nsCOMPtr<nsIFile> mozVoikkoLib;
+	rv = mozvoikkoHelper->GetComponentFile(getter_AddRefs(mozVoikkoLib));
+	if (NS_FAILED(rv))
+	{
+		logMessage("Failed to get component nsIFile. (rv = 0x%08x)\n", rv);
+		return NS_ERROR_UNEXPECTED;
+	}
+	
+	rv = mozVoikkoLib->GetNativePath(path);
+	if (NS_FAILED(rv))
+	{
+		logMessage("Failed to get path for mozVoikko helper component. (rv = 0x%08x)\n", rv);
+		return NS_ERROR_UNEXPECTED;
+	}
+	//logMessage("Path to mozvoikko helper component: %s", path.get());
+	
+	rv = mozVoikkoLib->Clone(file);
     if (NS_FAILED(rv))
     { 
         logMessage("%s: Failed to clone nsIFile", __FUNCTION__);
@@ -251,7 +245,6 @@ nsresult getMozVoikkoLibrary(nsIFile **file)
     return NS_OK;
 }
 
-#ifndef SYSTEM_LIBVOIKKO
 
 nsresult getMozVoikkoBaseDirs(nsIFile **libDir, nsIFile **dataDir)
 {
@@ -267,12 +260,10 @@ nsresult getMozVoikkoBaseDirs(nsIFile **libDir, nsIFile **dataDir)
     fullTargetName = mozvoikko_target_long;
 
     // Get path of mozvoikko shared library.
-    // mozVoikkoSpell::registerExtension() has saved it using nsICategory manager
-    // when registrating the extension
     rv = getMozVoikkoLibrary(getter_AddRefs(mozVoikkoLibName));
     if (NS_FAILED(rv))
     {
-        logMessage("mozvoikko: Failed to get nsICategoryManager entry");
+        logMessage("mozvoikko: Failed to get shared libraries location");
         return rv;
     }
 
